# -*- coding: utf-8 -*-
import cv2
import os
import glob
import time
import math
import json
import numpy as np

from tqdm import tqdm
from PIL import Image


####################################################################################
template_folder_path = '../input/template/'     # テンプレート画像へのパス 
input_folder_path = '../input/crosswalk/'       # 横断歩道画像へのパス
output_folder_path = '../output/'               # 出力フォルダへのパス

json_open = open('../input/config.json', 'r')
json_load = json.load(json_open)

# hyper parameter
gamma = json_load['gamma']                                  # ガンマ補正のガンマ値
num_morpho_iter = json_load['num_morpho_iter']              # モルフォロジー処理の繰り返し回数
s_thresh = json_load['s_thresh']                            # HSV色空間におけるSの閾値
dif_white_line = json_load['dif_white_line']                # 横断歩道の白線の幅を降順に並べたときに,信頼できる幅の差をどの程度とするか
standard_white_width = json_load['min_white_line']          # 横断歩道の左端と右端で，横断歩道の最大幅の何パーセントであれば横断歩道と認識するか
# width_ratio = json_load['width_ratio']                      # 横断歩道の白線の幅を最大値の何パーセントを横断歩道の幅の平均とするか
standard_white = json_load['std_white_line']                # 主成分分析により得られた線上のピクセル数の何パーセント以下まで，上下移動させるか
save_json = json_load['save_json']                          # 座標を出力するかどうか(true or false)
save_image = json_load['save_image']                        # 画像を出力するかどうか(true or false)
####################################################################################

# 全6種類のテンプレートマッチング手法のリスト
methods = ['cv2.TM_CCOEFF', 'cv2.TM_CCOEFF_NORMED', 'cv2.TM_CCORR',\
            'cv2.TM_CCORR_NORMED', 'cv2.TM_SQDIFF', 'cv2.TM_SQDIFF_NORMED']

class crosswalk:
    # 画像を読み込む関数
    def image_read(self, folder_path):
        '''
        Args:
            folder path => 画像フォルダへのパス
        Return：
            results => 要素が[ファイル名，画像]のリスト
        '''
        results = []
        for filepath in tqdm(glob.glob(folder_path + "*.JPG")):
            basename = os.path.splitext(os.path.basename(filepath))[0]
            img = cv2.imread(filepath)
            results.append([basename, img])
        return results

    # 横断歩道の検出から本数・摩耗率推定の一連の処理を行う関数
    def cw_detector(self, targets, templates, method):
        '''
        Args:
            targets => 検出を行う横断歩道画像(image_readで作成したもの)
            templates => テンプレート画像(image_readで作成したもの)
            method => methodsというリストからテンプレートマッチング手法を選択
        Return:
            なし
        '''
        with open('../output/results.txt', 'w') as f:
            f.write('')

        for i, target in tqdm(enumerate(targets), unit='枚'):
            max_value = []
            results = []
            out_txt = []

            y_m, x_m = target[1].shape[:2]
            out_txt.append(target[0])
            for template in templates:                                  # テンプレート画像の枚数だけ繰り返し
                res = cv2.matchTemplate(target[1], template[1], method) # テンプレートマッチング
                max_val, min_loc, max_loc = cv2.minMaxLoc(res)[1:]      # 最も類似度が高い画素値・画素位置と低い画素値・画素位置を取得
                max_value.append(max_val)
                if method in [cv2.TM_SQDIFF, cv2.TM_SQDIFF_NORMED]:     
                    top_left = min_loc
                else:
                    top_left = max_loc
                h, w = template[1].shape[:2]
                bottom_right = (top_left[0] + w, top_left[1] + h)
                results.append([top_left, bottom_right])                # 検出した横断歩道の左上と右下の座標をリストに追加
            # 複数のテンプレート画像の結果で，一番類似度の高いものを選択
            max_index = max_value.index(max(max_value))     

            if save_json or save_image:
                x1 = results[max_index][0][0]; y1 = results[max_index][0][1]    # 左上
                x2 = results[max_index][0][0]; y2 = results[max_index][1][1]    # 左下
                x3 = results[max_index][1][0]; y3 = results[max_index][1][1]    # 右下
                x4 = results[max_index][1][0]; y4 = results[max_index][0][1]    # 右上
                img_copy = target[1].copy()
                cv2.line(img_copy, (x1,y1),(x2,y2),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                cv2.line(img_copy, (x2,y2),(x3,y3),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                cv2.line(img_copy, (x3,y3),(x4,y4),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                cv2.line(img_copy, (x4,y4),(x1,y1),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                if save_image:
                    if not os.path.exists(output_folder_path + "img_detected"):
                        os.mkdir(output_folder_path + "img_detected/")
                    cv2.imwrite(output_folder_path + "img_detected/{0}.jpg".format(target[0]), img_copy)
                # if save_json:
                #     json_detected_cord = [{"x":x1,"y":y1},{"x":x4,"y":y4},{"x":x3,"y":y3},{"x":x2,"y":y2}]
                #     if not os.path.exists(output_folder_path + "json_detected"):
                #         os.mkdir(output_folder_path + "json_detected/")
                #     with open(output_folder_path + "json_detected/{0}.json".format(target[0]), "w") as f:
                #         json.dump(json_detected_cord, f, indent=4, ensure_ascii=False)

            # テンプレートマッチングで取得した領域に幅を持たせる
            x_min = (results[max_index][0][0]-x_m//4) if (results[max_index][0][0]-x_m//4) >= 0 else 0          
            y_min = (results[max_index][0][1]-y_m//100) if (results[max_index][0][1]-y_m//100) >= 0 else 0
            x_max = (results[max_index][1][0]+x_m//4) if (results[max_index][1][0]+x_m//4) < x_m else x_m-1
            y_max = (results[max_index][1][1]+y_m//100) if (results[max_index][1][1]+y_m//100) < y_m else y_m-1
            
            # テンプレートマッチングによる検出領域(幅を持たせた後)
            crop_image = target[1][y_min:y_max, x_min:x_max]

            # affine transformation
            img_resize = self.resize(x_m, y_m, crop_image)
            if save_image:
                if not os.path.exists(output_folder_path + "img_transformed"):
                    os.mkdir(output_folder_path + "img_transformed/")
                cv2.imwrite(output_folder_path + "img_transformed/{0}.jpg".format(target[0]), img_resize)

            # ガンマ補正
            table = np.array([((i / 255.0) ** (1 / gamma)) * 255 for i in np.arange(0, 256)]).astype("uint8")
            gamma_img = cv2.LUT(img_resize, table)

            # HSV色空間で閾値処理
            img_hsv = self.hsv(gamma_img)

            # グレースケールに変換し，モルフォロジー処理
            img_gray = self.gray(img_hsv)

            # 画像を二値化
            img_original_gray = cv2.threshold(cv2.cvtColor(img_hsv, cv2.COLOR_BGR2GRAY), 0, 255, cv2.THRESH_OTSU)[1]

            if save_image:
                # if not os.path.exists(output_folder_path + "crop_images/"):
                #     os.mkdir(output_folder_path + "crop_images/")
                # cv2.imwrite(output_folder_path + "crop_images/{0}.jpg".format(target[0]), crop_image)

                if not os.path.exists(output_folder_path + "img_hsv/"):
                    os.mkdir(output_folder_path + "img_hsv/")
                cv2.imwrite(output_folder_path + "img_hsv/{0}.jpg".format(target[0]), img_hsv)

                if not os.path.exists(output_folder_path + "img_gray/"):
                    os.mkdir(output_folder_path + "img_gray/")
                cv2.imwrite(output_folder_path + "img_gray/{0}.jpg".format(target[0]), img_gray)

            # 主成分分析(PCA)
            vec1, vec2 = self.PCA(img_gray, target[0])      # PCAによる2点の座標(両端)

            # 横断歩道の本数・摩耗率を取得
            cw_num,wear_rate = self.counter(img_gray, img_original_gray, target[0], vec1, vec2)
            if cw_num == 'error:1' or cw_num == 'error:2' or cw_num == 'error:3':
                out_txt += [' => 横断歩道本数:', '{0:3}'.format('ERROR'), '本', '　摩耗率:', '{0:6}'.format('ERROR'), '\n']
            else:
                out_txt += [' => 横断歩道本数:', '{0:3d}'.format(cw_num), '本', '　摩耗率:', '{0:6}'.format(round(wear_rate,2)), '%', '\n']
            
            with open('../output/results.txt', 'a') as f:
                f.writelines(out_txt)

            if save_json or save_image:
                x1 = x_min; y1 = y_min
                x2 = x_min; y2 = y_max
                x3 = x_max; y3 = y_max
                x4 = x_max; y4 = y_min
                if save_image:
                    if not os.path.exists(output_folder_path + "img_detected(after)"):
                        os.mkdir(output_folder_path + "img_detected(after)/")
                    cv2.line(target[1], (x1,y1),(x2,y2),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                    cv2.line(target[1], (x2,y2),(x3,y3),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                    cv2.line(target[1], (x3,y3),(x4,y4),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                    cv2.line(target[1], (x4,y4),(x1,y1),(0,0,255),thickness=8, lineType=cv2.LINE_8)
                    cv2.imwrite(output_folder_path + "img_detected(after)/{0}.jpg".format(target[0]), target[1])
                if save_json:
                    json_detected_cord = [{"x":x1,"y":y1},{"x":x4,"y":y4},{"x":x3,"y":y3},{"x":x2,"y":y2}]
                    if not os.path.exists(output_folder_path + "json_detected(after)"):
                        os.mkdir(output_folder_path + "json_detected(after)/")
                    with open(output_folder_path + "json_detected(after)/{0}.json".format(target[0]), "w") as f:
                        json.dump(json_detected_cord, f, indent=4, ensure_ascii=False)

    # 画像をリサイズする関数
    def resize(self, x, y, image):
        # 画像を(x × y)にリサイズ
        img_resize = cv2.resize(image, (x, y))
        return img_resize
    
    # 画像をグレースケールに変換して，モルフォロジー処理を行う関数
    def gray(self, image):
        gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)                  # グレースケール化
        thresh_image = cv2.threshold(gray, 0, 255, cv2.THRESH_OTSU)[1]  # 2値化

        kernel = np.ones((6, 6), np.uint8)
        img = thresh_image.copy()
        dilate = cv2.dilate(thresh_image, kernel)
        for _ in range(num_morpho_iter-1):
            dilate = cv2.dilate(dilate, kernel)
        erode = cv2.erode(dilate, kernel)
        for _ in range(num_morpho_iter-1):
            erode = cv2.erode(erode, kernel)
        return erode
    
    # 横断歩道の本数・摩耗率を推定する関数
    def counter(self, image, image_original, filename, vec1, vec2):
        # 横断歩道の本数・摩耗率推定
        x_min = min(vec1[0], vec2[0])       # それぞれの座標の最小値をx_min (PCAの結果の左端のx座標)
        x_max = max(vec1[0], vec2[0])       # それぞれの座標の最大値をx_max (PCAの結果の右端のx座標)
        pixel_val = []
        pixel_num = []
        white_len = []          # 横断歩道の連続幅(白色ピクセルの連結幅)のリスト
        x_point_left = []       # 連結幅の左端の座標のリスト
        x_point_right = []      # 連結幅の右端の座標のリスト
        counter = 0             
        flag = 0    # 最初の白領域用
        flagg = 0   # 0-255の切り替わり用
        # dist = 0    # 白領域の距離
        left = None
        right = None
        left_temp = None
        coordinate_list = []    # 連結幅の中心の座標のリスト

        # PCAの結果の直線のx_minにおけるy座標 (PCAの結果の左端のy座標)
        if (vec2[0]-vec1[0]) == 0:
            y_num = int((vec2[1]-vec1[1])*(x_min-vec1[0])/(1e-3) + vec1[1])
        else:
            y_num = int((vec2[1]-vec1[1])*(x_min-vec1[0])/(vec2[0]-vec1[0]) + vec1[1])

        # PCAの結果の直線を，左端から右端まで見ていき，輝度値の変化のグラフ(値)を取得
        for x in range(x_min,x_max+1):
            if (vec2[0]-vec1[0]) == 0:
                y_num = int((vec2[1]-vec1[1])*(x-vec1[0])/(1e-3) + vec1[1])
            else:
                y_num = int((vec2[1]-vec1[1])*(x-vec1[0])/(vec2[0]-vec1[0]) + vec1[1])
            if image[y_num, x] != 255:          # グラフの端の白い部分を削除
                flag = 1
            pixel_val.append(image[y_num, x])
            pixel_num.append(x+1)
            if flag == 1:
                if image[y_num, x] == 255 and flagg == 1:
                    continue
                elif image[y_num, x] == 255:
                    if left == None:
                        left = (x,y_num)
                    if flagg == 0:
                        left_temp = (x,y_num)
                    flagg = 1
                elif left_temp != None:
                    right_temp = (x,y_num)
                    dist = math.sqrt((right_temp[0]-left_temp[0])**2 + (right_temp[1]-left_temp[1])**2)
                    white_len.append(dist)
                    coordinate_list.append(((left_temp[0]+right_temp[0])/2,(left_temp[1]+right_temp[1])/2))
                    x_point_left.append(left_temp[0])
                    x_point_right.append(right_temp[0])
                    left_temp = None
                else:
                    flagg = 0

        # 横断歩道の幅を降順に並べ替え
        white_len = np.array(white_len)
        ind_sorted = np.argsort(-white_len)
        white_len_ori = white_len
        white_len = white_len[ind_sorted]
        coordinate_list = np.array(coordinate_list)
        coordinate_list = coordinate_list[ind_sorted]
        x_point_left = np.array(x_point_left)
        x_point_right = np.array(x_point_right)

        if len(white_len) == 0 or len(white_len) == 1:
            # 横断歩道が2つ以上検出されなかった場合
            return 'error:1', 'error:1'

        white_width = 0
        # 横断歩道の幅を降順に並べたときに,幅の差が dif_white_line 以下であればその値を横断歩道の最大幅として扱う(READMEのハイパーパラメータ参照)
        # white_width => 横断歩道の最大幅
        
        white_line_list = []
        for i in range(0, len(white_len)-1):
            if white_len[i] * dif_white_line <= white_len[i+1]:
                if white_width == 0:
                    white_width = white_len[i]
                    index_width = i
                white_line_list.append(white_len[i])
                #break
        if white_width == 0:
            white_width = white_len[0]
        
        if len(white_line_list) != 0:
            ave_white_line = sum(white_line_list)/len(white_line_list)
        else:
            ave_white_line = white_width
        
        # 横断歩道の左端から見ていき，横断歩道の端の座標を取得
        for i in range(len(white_len_ori)):
            if white_len_ori[i] > white_width*standard_white_width:
                if (vec2[0]-vec1[0]) == 0:
                    left = (x_point_left[i],int((vec2[1]-vec1[1])*(x_point_left[i]-vec1[0])/(1e-3) + vec1[1]))
                else:
                    left = (x_point_left[i],int((vec2[1]-vec1[1])*(x_point_left[i]-vec1[0])/(vec2[0]-vec1[0]) + vec1[1]))
                break
        for i in reversed(range(0,len(white_len_ori))):
            if white_len_ori[i] > white_width*standard_white_width:
                if (vec2[0]-vec1[0]) == 0:
                    right = (x_point_right[i],int((vec2[1]-vec1[1])*(x_point_right[i]-vec1[0])/(1e-3) + vec1[1]))
                else:
                    right = (x_point_right[i],int((vec2[1]-vec1[1])*(x_point_right[i]-vec1[0])/(vec2[0]-vec1[0]) + vec1[1]))
                break
        ##################################################################################################################################################
        if left == None or right == None:
            # 横断歩道を領域を取得できなかった場合
            return 'error:2', 'error:2'
        crosswalk_dist = math.sqrt((right[0]-left[0])**2 + (right[1]-left[1])**2)       # 横断歩道の幅を計算

        if save_image:
            if not os.path.exists(output_folder_path + "img_width/"):
                os.mkdir(output_folder_path + "img_width/")
            img = cv2.imread(output_folder_path + "img_gray/{0}.jpg".format(filename)) # RGB Image
            cv2.circle(img, left, 15, (255, 0, 0), thickness=-1)
            cv2.circle(img, right, 15, (0, 0, 255), thickness=-1)
            cv2.imwrite(output_folder_path + "img_width/{0}.jpg".format(filename), img)
        
        counter = int(round(crosswalk_dist / (2*ave_white_line)) +1)
        
        # 最大幅の重心の座標
        x_coordinate = int(coordinate_list[index_width][0])     
        y_coordinate = int(coordinate_list[index_width][1])
        
        # 主成分分析の結果の直線をスライドさせていき，切れ目を探す
        x_max_,y_max = image.shape[::-1]
        region_origin = 0
        for y in range(0,y_coordinate+1,1):
            region = 0
            for x in range(x_min, x_max):
                if y == 0:
                    if (vec2[0]-vec1[0]) == 0:
                        if image[int((vec2[1]-vec1[1])*(x-vec1[0])/(1e-3) + vec1[1]), x] == 255:
                            region_origin += 1
                    else:
                        if image[int((vec2[1]-vec1[1])*(x-vec1[0])/(vec2[0]-vec1[0]) + vec1[1]), x] == 255:
                            region_origin += 1
                else:
                    if (vec2[0]-vec1[0]) == 0:
                        if int((vec2[1]-vec1[1])*(x-vec1[0])/(1e-3) + vec1[1])-y < 0:
                            break
                        if image[int((vec2[1]-vec1[1])*(x-vec1[0])/(1e-3) + vec1[1])-y, x] == 255:
                            region += 1
                    else:
                        if int((vec2[1]-vec1[1])*(x-vec1[0])/(vec2[0]-vec1[0]) + vec1[1])-y < 0:
                            break
                        if image[int((vec2[1]-vec1[1])*(x-vec1[0])/(vec2[0]-vec1[0]) + vec1[1])-y, x] == 255:
                            region += 1
            if y == 0:
                continue
            elif region <= region_origin*standard_white:
                white_y_min = y_coordinate - y
                break
            if y == y_coordinate:
                white_y_min = 0
        # 主成分分析の結果の直線をスライドさせていき，切れ目を探す
        for y in range(1,(y_max-y_coordinate),1):
            region = 0
            for x in range(x_min, x_max):
                if (vec2[0]-vec1[0]) == 0:
                    if int((vec2[1]-vec1[1])*(x-vec1[0])/(1e-3) + vec1[1])+y > y_max-1:
                        break
                    if image[int((vec2[1]-vec1[1])*(x-vec1[0])/(1e-3) + vec1[1])+y, x] == 255:
                        region += 1
                else:
                    if int((vec2[1]-vec1[1])*(x-vec1[0])/(vec2[0]-vec1[0]) + vec1[1])+y > y_max-1:
                        break
                    if image[int((vec2[1]-vec1[1])*(x-vec1[0])/(vec2[0]-vec1[0]) + vec1[1])+y, x] == 255:
                        region += 1
            if region <= region_origin*standard_white:
                white_y_max = y_coordinate + y
                break
            if y == (y_max-y_coordinate-1):
                white_y_max = y_max-1

        if (vec2[0]-vec1[0]) == 0:
            theta = math.degrees(math.atan((vec2[1]-vec1[1])/(1e-3)))
        else:
            theta = math.degrees(math.atan((vec2[1]-vec1[1])/(vec2[0]-vec1[0])))
        rows, cols = image.shape
        M = cv2.getRotationMatrix2D((cols/2,rows/2),theta,1)
        dst = cv2.warpAffine(image_original,M,(cols,rows))
        
        # 回転行列算出関数
        def rotate_x(deg):
            # degreeをradianに変換
            r = np.radians(deg)
            C = np.cos(r)
            S = np.sin(r)
            # x軸周りの回転行列
            R_x = np.matrix((
                (C, -S),
                (S, C)
            ))
            return R_x
        
        Rx = rotate_x(-theta)
        left_rt = np.dot(Rx,(left[0]-cols/2,left[1]-rows/2))
        left_rt = left_rt + (cols/2,rows/2)
        left_rt = tuple(np.squeeze(np.asarray(left_rt, dtype='int64')))
        right_rt = np.dot(Rx,(right[0]-cols/2,right[1]-rows/2))
        right_rt = right_rt + (cols/2,rows/2)
        right_rt = tuple(np.squeeze(np.asarray(right_rt, dtype='int64')))

        # 上下左右に20%幅を広げる
        y_min_trim = left_rt[1]-(y_coordinate-white_y_min) - int(((left_rt[1]+(white_y_max-y_coordinate)-(left_rt[1]-(y_coordinate-white_y_min)))//2)*0.4)
        y_max_trim = left_rt[1]+(white_y_max-y_coordinate) + int(((left_rt[1]+(white_y_max-y_coordinate)-(left_rt[1]-(y_coordinate-white_y_min)))//2)*0.4)
        x_min_trim = left_rt[0] - int(((right_rt[0]-left_rt[0])//2)*0.2)
        x_max_trim = right_rt[0] + int(((right_rt[0]-left_rt[0])//2)*0.2)
        if x_min_trim < 0:
            x_min_trim = 0
        if x_max_trim >= x_max_:
            x_max_trim = x_max_ - 1
        if y_min_trim < 0:
            y_min_trim = 0
        if y_max_trim >= y_max:
            y_max_trim = y_max - 1
        img_crosswalk = dst[y_min_trim:y_max_trim, x_min_trim:x_max_trim]

        # if save_image:
        #     if not os.path.exists(output_folder_path + "img_cw/"):
        #         os.mkdir(output_folder_path + "img_cw/")
        #     cv2.imwrite(output_folder_path + "img_cw/{0}.jpg".format(filename), img_crosswalk)
        
        # white_area_gt = white_width * (white_y_max - white_y_min) * (counter-1)
        white_area_gt = ave_white_line * (white_y_max - white_y_min) * (counter)
        if white_area_gt == 0:
            # 横断歩道領域がない場合
            return 'error:3', 'error:3'
        white_area_img = np.where(img_crosswalk==255)
        if white_area_gt < len(white_area_img[0]):
            wear_rate = 0
        else:
            wear_rate = (1-(len(white_area_img[0])/white_area_gt)) * 100
        
        # if save:
        #     img = cv2.imread(output_folder_path + "img_gray/{0}.jpg".format(filename)) # RGB Image
        #     cv2.circle(img, (800,white_y_min), 15, (255, 0, 0), thickness=-1)
        #     cv2.circle(img, (800,white_y_max), 15, (0, 0, 255), thickness=-1)
        #     cv2.circle(img, (x_coordinate, y_coordinate), 15, (0, 255, 0), thickness=-1)
        #     if not os.path.exists(output_folder_path + "point/"):
        #         os.mkdir(output_folder_path + "point/")
        #     cv2.imwrite(output_folder_path + "point/{0}.jpg".format(filename), img)
            
        return counter,wear_rate
    
    # HSV色空間における彩度の閾値処理を行う関数
    def hsv(self, image):
        img_hsv = cv2.cvtColor(image, cv2.COLOR_BGR2HSV)
        img = image.copy()
        s = img_hsv[:,:,1]
        y, x = img_hsv.shape[:2]
        b, g, r = cv2.split(img)
        b_mean = b.mean()
        g_mean = g.mean()
        r_mean = r.mean()
        black = np.where(s > s_thresh)
        for i in range(len(black[0])):
            img[black[0][i]][black[1][i]] = (b_mean, g_mean, r_mean)
        
        return img
    
    # 画像の白色ピクセルの座標を用いて，主成分分析(PCA)を行う関数
    def PCA(self, image, filename):

        white_area = np.where(image==255)
        lis = []
        for i in range(len(white_area[0])):
            white = np.array([white_area[1][i], white_area[0][i]])
            lis.append(white)
        white_data = np.stack(lis,axis=0)
        x_ = white_data.mean(axis=0)
        standardscaler = (white_data - x_)/white_data.std(ddof=0)
        v_cov = np.dot(white_data.T,white_data)/white_data.shape[0]-np.dot(x_[:,np.newaxis],x_[np.newaxis,:])

        eig, eig_vec = np.linalg.eig(v_cov)

        v1 = eig_vec[:,0]
        center = white_data.mean(axis=0)          # 重心を計算
        
        v1 = v1*100 + center
        
        h, w = image.shape[:2]                                                                        
        
        # PCAの結果の，左端の座標(pt1)と右端の座標(pt2)を算出
        if (v1[0]-center[0]) == 0:
            y1_start = (v1[1]-center[1])*(0-center[0])/(1e-3) + center[1]        # 始点のy座標    
            y1_end = (v1[1]-center[1])*((w-1)-center[0])/(1e-3) + center[1]      # 終点のy座標 
        else:
            y1_start = (v1[1]-center[1])*(0-center[0])/(v1[0]-center[0]) + center[1]        # 始点のy座標    
            y1_end = (v1[1]-center[1])*((w-1)-center[0])/(v1[0]-center[0]) + center[1]      # 終点のy座標 

        if (v1[1]-center[1]) == 0:
            if y1_start < 0:                                                                                
                x1_start = (0-center[1])*(v1[0]-center[0])/(1e-3) + center[0]                    
                pt1 = (int(x1_start), 0)
            elif y1_start >= h:                                                                             
                x1_start = ((h-1)-center[1])*(v1[0]-center[0])/(1e-3) + center[0]                
                pt1 = (int(x1_start), h-1)                                                                  
            else:                                                                                            
                pt1 = (0, int(y1_start))                                                                    
                                                                                                            
            if y1_end < 0:                                                                                  
                x1_end = (0-center[1])*(v1[0]-center[0])/(1e-3) + center[0]                      
                pt2 = (int(x1_end), 0)                                                                      
            elif y1_end >= h:                                                                               
                x1_end = ((h-1)-center[1])*(v1[0]-center[0])/(1e-3) + center[0]                  
                pt2 = (int(x1_end), h-1)                                                                    
            else:                                                                                           
                pt2 = (w-1, int(y1_end))
        else:
            if y1_start < 0:                                                                                
                x1_start = (0-center[1])*(v1[0]-center[0])/(v1[1]-center[1]) + center[0]                    
                pt1 = (int(x1_start), 0)
            elif y1_start >= h:                                                                             
                x1_start = ((h-1)-center[1])*(v1[0]-center[0])/(v1[1]-center[1]) + center[0]                
                pt1 = (int(x1_start), h-1)                                                                  
            else:                                                                                            
                pt1 = (0, int(y1_start))                                                                    
                                                                                                            
            if y1_end < 0:
                x1_end = (0-center[1])*(v1[0]-center[0])/(v1[1]-center[1]) + center[0]
                pt2 = (int(x1_end), 0)
            elif y1_end >= h:                                                                               
                x1_end = ((h-1)-center[1])*(v1[0]-center[0])/(v1[1]-center[1]) + center[0]                  
                pt2 = (int(x1_end), h-1)
            else:
                pt2 = (w-1, int(y1_end))
        
        if save_image:
            img = cv2.imread(output_folder_path + "img_gray/{0}.jpg".format(filename)) # RGB Image
            cv2.line(img, pt1, pt2, (0, 255, 0), thickness=50)
            if not os.path.exists(output_folder_path + "img_PCA/"):
                os.mkdir(output_folder_path + "img_PCA/")
            cv2.imwrite(output_folder_path + "img_PCA/{0}.jpg".format(filename), img)
        # if save_json:
        #     json_pca_cord = [{"x":pt1[0],"y":pt1[1]},{"x":pt2[0],"y":pt2[1]}]
        #     if not os.path.exists(output_folder_path + "json_PCA"):
        #         os.mkdir(output_folder_path + "json_PCA/")
        #     with open(output_folder_path + "json_PCA/{0}.json".format(filename), "w") as f:
        #         json.dump(json_pca_cord, f, indent=4, ensure_ascii=False)
        return pt1, pt2
    
def main():
    cw = crosswalk()

    # input images loading
    print("\n[input images loading...]")
    start = time.time()
    print("crosswalk images...")
    targets = cw.image_read(input_folder_path)
    print("template images...")
    templates = cw.image_read(template_folder_path)
    print("[input images loaded(time taken:%.2f s)!]"%(time.time()-start))

    # crosswalk detecting and counting
    print("\n[crosswalk detecting and counting...]")
    start = time.time()
    cw.cw_detector(targets, templates, eval(methods[1]))
    print("[crosswalk detected(time taken:%.2f s)!]"%(time.time()-start))
    
if __name__ == '__main__':
    main()
